package com.community.pocket.ui.main.ui.forum.news;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.SearchView;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;

import com.community.pocket.R;
import com.community.pocket.ui.main.ui.forum.ForumPost;
import com.community.pocket.ui.main.ui.share.Response;

import org.xutils.view.annotation.ContentView;
import org.xutils.view.annotation.ViewInject;

/**
 * 最新帖子
 */
@ContentView(R.layout.forum_new_fragment)
public class ForumNewFragment extends ForumPost {

    private ForumNewViewModel viewModel;

    @ViewInject(R.id.search)
    private SearchView searchView;

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        viewModel = new ViewModelProvider(this, new ViewModelProvider.NewInstanceFactory()).get(ForumNewViewModel.class);

        viewModel.loadForumNew(1, searchView.getQuery().toString());

        searchView.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextSubmit(String query) {
                if (!query.isEmpty()) {
                    viewModel.loadForumNew(1, searchView.getQuery().toString());
                    return true;
                } else {
                    return false;
                }

            }

            @Override
            public boolean onQueryTextChange(String newText) {
                return false;
            }
        });

        //监听最新帖子请求状态
        viewModel.getForumNewResponse().observe(getViewLifecycleOwner(), new Observer<ForumNewResponse>() {
            @Override
            public void onChanged(ForumNewResponse forumNewResponse) {
                if (forumNewResponse == null) {
                    return;
                }

                forumNewResponse.toast(getContext());

                if (forumNewResponse.getResult() == Response.Result.OK) {
                    loadPost(forumNewResponse.getForumList());
                }
            }
        });
    }

    @Override
    protected int showAuthor() {
        return View.VISIBLE;
    }

    @Override
    protected int showStatus() {
        return View.GONE;
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        viewModel.loadForumNew(1, searchView.getQuery().toString());
    }
}
